@@ -13,6 +13,7 @@ import logging
 import time
 import json
 import uuid
+import re
 from functools import lru_cache
 from fastapi import FastAPI, Request, HTTPException
 from fastapi.middleware.cors import CORSMiddleware
@@ -44,6 +45,39 @@ class InsightType(Enum):
     HISTORICAL_SEMANTIC = "historical-semantic"
     HYBRID = "hybrid"
 
+class DataPreprocessor:
+    """Preprocesses incoming data by cleaning, normalizing, and enriching text"""
+    def __init__(self, config: Optional[Dict[str, Any]] = None):
+        self.config = config or {}
+        self.remove_special_chars = self.config.get('remove_special_chars', True)
+        self.lowercase = self.config.get('lowercase', True)
+        self.remove_extra_spaces = self.config.get('remove_extra_spaces', True)
+        self.special_char_pattern = re.compile(r'[^\w\s.,!?]')  # Keep alphanumeric, spaces, and basic punctuation
+    
+    def preprocess(self, text: str) -> str:
+        """Apply preprocessing transformations to the input text"""
+        if not text or not isinstance(text, str):
+            return text
+        
+        try:
+            processed = text
+            # Step 1: Lowercase
+            if self.lowercase:
+                processed = processed.lower()
+            
+            # Step 2: Remove special characters
+            if self.remove_special_chars:
+                processed = self.special_char_pattern.sub('', processed)
+            
+            # Step 3: Remove extra spaces
+            if self.remove_extra_spaces:
+                processed = ' '.join(processed.split())
+            
+            return processed.strip()
+        except Exception as e:
+            logger.warning(f"Preprocessing failed for text: {text[:50]}... Error: {str(e)}")
+            return text
+
 class ContextChainConfig(BaseModel):
     """Configuration for ContextChain"""
     max_tokens: int = 4096
@@ -63,6 +97,7 @@ class ContextChainConfig(BaseModel):
     structured_indicators: List[str] = ["forecast", "prediction", "metric", "kpi", "revenue", "sales", "result", "average", "trend"]
     contextual_indicators: List[str] = ["analysis", "insight", "summary", "context", "document"]
     default_destination: DataDestination = DataDestination.MONGODB
+    preprocessor_config: Optional[Dict[str, Any]] = None  # New: Config for DataPreprocessor
 
     class Config:
         arbitrary_types_allowed = True
@@ -118,6 +153,7 @@ class ContextChain:
         self.vector_store = HybridVectorStore(config.vector_config or VectorStoreConfig())
         self.dag_engine = DAGEngine(config.max_parallel_tasks)
         self.storage = IntelligentStorage(mongo_uri=config.mongo_uri) if config.enable_mongo else None
+        self.preprocessor = DataPreprocessor(config.preprocessor_config or {})  # New: Initialize DataPreprocessor
         self.performance_stats = {
             'total_requests': 0,
             'successful_requests': 0,
@@ -151,7 +187,7 @@ class ContextChain:
 
     def _classify_query(self, query: str) -> str:
         """Classify query type for routing (structured, semantic, or hybrid)"""
-        query_lower = query.lower()
+        query_lower = self.preprocessor.preprocess(query).lower()  # Preprocess query
         if any(indicator in query_lower for indicator in self.config.structured_indicators):
             return "structured"
         elif any(indicator in query_lower for indicator in self.config.contextual_indicators):
@@ -169,15 +205,16 @@ class ContextChain:
                 results = [{"metric": "average_sales", "value": 12345.67, "period": "2025-Q3"}]
             elif "trend" in query.lower():
                 results = [{"metric": "sales_trend", "value": 5.2, "unit": "percent", "period": "2024-2025"}]
-            return [
+            processed_results = [
                 {
-                    "content": str(result),
+                    "content": self.preprocessor.preprocess(str(result)),  # Preprocess content
                     "source_id": f"postgres_{uuid.uuid4()}",
                     "metadata": {"type": "structured", "metric": result.get("metric")},
                     "score": 1.0,
                     "embedding_quality": 1.0
                 } for result in results
             ]
+            return processed_results
         except Exception as e:
             logger.warning(f"Structured query failed: {str(e)}, returning empty results")
             return []
@@ -193,6 +230,8 @@ class ContextChain:
         data_routing = {}
         
         try:
+            # Preprocess query
+            query = self.preprocessor.preprocess(query)
             await self._validate_input(query, documents)
             processing_steps.append({
                 'step': 'input_validation',
@@ -200,6 +239,13 @@ class ContextChain:
                 'timestamp': datetime.utcnow()
             })
             
+            # Preprocess documents if provided
+            if documents:
+                documents = [
+                    {**doc, "content": self.preprocessor.preprocess(doc["content"])}
+                    for doc in documents
+                ]
+            
             execution_context = ExecutionContext(session_id=session_id, query=query, raw_data={'documents': documents or []})
             
             complexity = self.acba.complexity_assessor.assess_complexity(query)
@@ -337,10 +383,12 @@ class ContextChain:
     async def _retrieve_documents(self, query: str, context: ExecutionContext) -> List[Dict[str, Any]]:
         """Retrieve relevant documents from vector store"""
         try:
+            # Preprocess query before searching
+            query = self.preprocessor.preprocess(query)
             search_results = await self.vector_store.search(query=query, k=5)
             return [
                 {
-                    'content': result.content,
+                    'content': self.preprocessor.preprocess(result.content),  # Preprocess retrieved content
                     'source_id': result.source_id,
                     'score': result.fusion_score,
                     'metadata': result.metadata,
@@ -430,7 +478,12 @@ class ContextChain:
     async def index_documents(self, documents: List[Dict[str, Any]]) -> Dict[str, Any]:
         """Index documents in vector store"""
         try:
-            doc_ids = await self.vector_store.index_documents(documents)
+            # Preprocess documents before indexing
+            processed_docs = [
+                {**doc, "content": self.preprocessor.preprocess(doc["content"])}
+                for doc in documents
+            ]
+            doc_ids = await self.vector_store.index_documents(processed_docs)
             stats = self.vector_store.get_performance_stats()
             return {
                 'status': 'success',
@@ -462,6 +515,7 @@ class ContextChain:
 
     def decide_data_destination(self, data_type: str, content: str, metadata: Optional[Dict] = None) -> DataDestination:
         """Intelligent routing decision for metadata storage (fallback)"""
+        content = self.preprocessor.preprocess(content)  # Preprocess content
         content_lower = content.lower()
         if data_type in self.config.structured_indicators or any(indicator in content_lower for indicator in self.config.structured_indicators):
             return DataDestination.POSTGRESQL
@@ -473,8 +527,8 @@ class ContextChain:
     async def store_business_data(self, data_type: str, data: Any, metadata: Optional[Dict] = None,
                                  destination: Optional[str] = None) -> Dict[str, str]:
         """Store business data with user-specified destination or fallback routing"""
-        import json
         content = json.dumps(data) if not isinstance(data, str) else data
+        content = self.preprocessor.preprocess(content)  # Preprocess content
         routing = {}
         
         if destination:
@@ -506,11 +560,14 @@ class ContextChain:
 
     async def search_context(self, query: str, k: int = 5, data_type: Optional[str] = None) -> List[Dict[str, Any]]:
         """Search stored context in vector store"""
+        query = self.preprocessor.preprocess(query)  # Preprocess query
         return await self.vector_store.search(query=query, k=k, metadata_filter={"data_type": data_type} if data_type else None)
 
     async def submit_feedback(self, session_id: str, rating: int, comments: Optional[str] = None) -> Dict[str, str]:
         """Store feedback for a session"""
         try:
+            if comments:
+                comments = self.preprocessor.preprocess(comments)  # Preprocess comments
             if not self.storage:
                 raise ValueError("MongoDB storage not enabled for feedback")
             if not 1 <= rating <= 5:
