@@ -227,7 +227,7 @@ class OllamaLLM(BaseLLMClient):
             raise ImportError("httpx library not installed. Run: pip install httpx")
         self.client = httpx.AsyncClient(
             base_url=config.api_base or "http://localhost:11434",
-            timeout=config.timeout
+            timeout=120
         )
     
     async def generate(self, prompt: str, max_tokens: int, temperature: float, **kwargs) -> GenerationResult:
@@ -244,7 +244,10 @@ class OllamaLLM(BaseLLMClient):
             response = await self.client.post("/api/generate", json=payload)
             response.raise_for_status()
             data = response.json()
-            content = data.get("response", "").strip()
+            if isinstance(data.get("response"), list):
+                content = "".join(chunk.get("response", "") for chunk in data["response"]).strip()
+            else:
+                content = str(data.get("response", "")).strip()
             return GenerationResult(
                 content=content,
                 tokens_used=data.get("eval_count", 0) + data.get("prompt_eval_count", 0),
